/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Injectable} from "@angular/core";
import {HttpClient} from "@angular/common/http";
import {Observable} from "rxjs";
import {CapabilityRequest} from "../model/capability-request";
import {CapabilityVersion} from "../model/capability-version";
import {Staff} from "../model/staff";
import {JsonObject} from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import {ActivatedRoute, Router} from "@angular/router";

@Injectable({
  providedIn: "root",
})
export class CapabilityRequestService {
  baseUrl = "/workspaces";

  constructor(
    private httpClient: HttpClient,
    private router: Router,
    private route: ActivatedRoute,
  ) {
  }

  /**
   * Returns URL for GETting capability request definition
   * @param id  Capability request ID
   * @return string  The "GET capability request" HTTP url
   */
  public getCapabilityRequestURL(id: number): string {
    return `capability/request/${id}`;
  }

  /**
   * Gets JSON representation of a capability request given an ID
   * @param id  The ID of the request
   * @return Observable<CapabilityRequest>  Result of REST caall to get the request
   */
  public getCapabilityRequest(id: number): Observable<CapabilityRequest> {
    return this.httpClient.get<CapabilityRequest>(this.getCapabilityRequestURL(id));
  }

  /**
   * Gets JSON representation of a list of active capability requests for a given capability
   * @param capabilityName  The name of the capability
   */
  public getActiveCapabilityRequests(capabilityName: string): Observable<JsonObject> {
    const url = `capability/${capabilityName}/request`;
    return this.httpClient.get<JsonObject>(url);
  }

  /**
   * Gets URL to retrieve a list of active capability requests for a given capability
   * @param capabilityName  The name of the capability
   */
  public getActiveCapabilityRequestsUrl(capabilityName: string, filters: Array<any>): string {
    var qs = Object.keys(filters).map(key => {
      if (Array.isArray(filters[key])) {
        let arrayKeys=[]
        for(var i=0;i < filters[key].length; i++) {
          arrayKeys.push(key + '=' + filters[key][i])
        }
        return arrayKeys.join('&')
      }
      return key + '=' + filters[key]
    }).join('&')
    return `capability/${capabilityName}/request?${qs}`;
  }

  /**
   * Gets JSON representation of a list of active capability requests for a given capability
   * @param capabilityName  The name of the capability
   */
  public getCreatedCapabilityRequests(capabilityName: string): Observable<JsonObject> {
    const url = `capability/${capabilityName}/created-requests`;
    return this.httpClient.get<JsonObject>(url);
  }

  /**
   * Get all versions of a capability request
   * @param requestId ID of the parent request
   * @return Array<CapabilityVersion> Result of REST call to retrieve all versions
   */
  public getAllVersions(requestId: string): Observable<Array<CapabilityVersion>> {
    const getVersions = `capability/request/${requestId}/versions`;
    return this.httpClient.get<any>(getVersions);
  }

  public getSpecificVersion(request_id: string, version_id: number): Observable<JsonObject> {
    const url = `capability/request/${request_id}/version/${version_id}`;
    return this.httpClient.get<JsonObject>(url);
  }

  /**
   * Returns URL for GETting the latest version of a capability request
   * @param id  Capability request ID
   * @return string  The "GET latest version" HTTP url
   */
  public getLatestVersionURL(id: number): string {
    return `capability/request/${id}/version/latest`;
  }

  /**
   * Get latest version for capability request with given ID
   * @param id  ID of request to get version for
   * @return Observable<CapabilityVersion>  Result of REST call to get the version
   */
  public getLatestVersion(id: number): Observable<CapabilityVersion> {
    return this.httpClient.get<CapabilityVersion>(this.getLatestVersionURL(id));
  }

  /**
   * Add file to selected capability request version
   * @param requestId  ID of request whose latest version will receive the file
   * @param filename  Name of file
   * @param content  Content of file
   * @return  Observable<string>  Result of REST call to add the file
   * @private
   */
  updateVersionFile(requestId: number, filename: string, content: string): Observable<string> {
    const addFileUrl = `capability/request/${requestId}/version/latest/files/${filename}`;
    return this.httpClient.put<string>(addFileUrl, content);
  }

  /**
   * QA Pass this request by POSTing URL to REST endpoint
   *
   * @param requestId ID of request whose capability execution is to be passed
   * @param version The capability version that begat this workflow request
   */
  public sendQaPassRequest(requestId: number, version: number): Observable<string> {
    const qaPassUrl = `workflows/requests/${requestId}/qa/qa_pass`;
    return this.httpClient.post<string>(qaPassUrl, {capability_version: version});
  }

  /**
   * QA Fail this request by POSTing URL to REST endpoint
   *
   * @param requestId ID of request whose capability execution is to be passed
   * @param version The capability version that begat this workflow request
   */
  public sendQaFailRequest(requestId: number, version: number): Observable<string> {
    const qaFailUrl = `workflows/requests/${requestId}/qa/qa_fail`;
    return this.httpClient.post<string>(qaFailUrl, {capability_version: version});
  }

  /**
   * Abandon the whole capability request by sending a message via the qa_announce functionality
   *
   * @param requestId ID of the request to be sealed and fully failed
   * @param version The capability version from which the request was sent
   */
  public sendQaAbandonRequest(requestId: number, version: number): Observable<string> {
    const qaAbandonUrl = `workflows/requests/${requestId}/qa/qa_abandon`;
    return this.httpClient.post<string>(qaAbandonUrl, {capability_version: version});
  }

  /**
   * Stage 2 Pass (i.e. proceed to ingestion) this request by POSTing URL to REST endpoint
   *
   * @param requestId ID of request whose capability execution is to be passed
   * @param version The capability version that begat this workflow request
   */
  public sendStage2QaPassRequest(requestId: number, version: number): Observable<string> {
    const stage2PassUrl = `workflows/requests/${requestId}/qa/stage_2_pass`;
    return this.httpClient.post<string>(stage2PassUrl, { capability_version: version });
  }

  /**
   * Qa Revisit (i.e. pass back to DA) this request by POSTing URL to REST endpoint
   *
   * @param requestId ID of request whose capability execution is to be passed
   * @param version The capability version that begat this workflow request
   */
  public sendQaRevisitRequest(requestId: number, version: number): Observable<string> {
    const revisitUrl = `workflows/requests/${requestId}/qa/qa_revisit`;
    return this.httpClient.post<string>(revisitUrl, { capability_version: version });
  }

  /**
   * Close this request by POSTing the ID to a REST endpoint.
   *
   * @param requestId ID of the capability request to close
   */
  public sendCloseRequest(requestId: string): Observable<string> {
    const closeUrl = `capability/request/${requestId}/close`;
    return this.httpClient.post<string>(closeUrl, {responseType: 'text'});
  }

  /**
   * Create a new version of the specified capability request with specified settings
   * @param request_id  ID of request to add new version to
   * @param parametersString  Parameters of the new version as a string
   * @param inputFiles  Input files to give to the new version
   */
  public createNewVersion(
    request_id: number,
    parametersString: string,
    inputFiles: FileList,
  ): Observable<CapabilityVersion> {
    const parameters = JSON.parse(parametersString);
    const newVersionUrl = `capability/request/${request_id}/version/create`;

    return this.httpClient.post<CapabilityVersion>(
      newVersionUrl,
      JSON.stringify({parameters: parameters}),
    );
  }

  /**
   * Add all files from a file list to the latest version of a given request
   * @param requestId  ID for request to add files to
   * @param inputFileList  List of files to be added
   * @return Array<string>  List of filenames that were successfully added
   */
  public addFilesToVersion(requestId: string, inputFileList: FileList) {
    const addFileObservable = {
      next: (response) => {
        console.log("Successfully added file to version.");
      },
      error: (error) => console.error("Error when adding file to version:" + error),
    };

    for (let i = 0; i < inputFileList.length; i++) {
      const currentFile = inputFileList.item(i);
      const filename = currentFile.name;
      currentFile
        .text()
        .then((content) =>
          this.updateVersionFile(parseInt(requestId), filename, content).subscribe(addFileObservable),
        );
    }
  }

  /**
   * Redirect to status page of capability request with given ID
   * @param requestId  Given request ID
   */
  public redirectToRequestStatusPage(requestId: string, shouldOpenInNewTab: boolean = true) {
    const statusPageUrl = this.router.serializeUrl(
      this.router.createUrlTree([`${this.baseUrl}/request-status/${requestId}`], {
        relativeTo: this.route,
      }),
    );
    // this.router.navigate([`request-status/${requestId}`], { relativeTo: this.route });
    if (shouldOpenInNewTab) {
      window.open(statusPageUrl, "_blank");
    } else {
      this.router.navigate([statusPageUrl]);
    }
  }

  /**
   * Toggles SRDP Compatibility to true and false
   * @param request_id  Given request ID
   * @param version_id  Given version ID
   * @param isAlreadySRDP The current status of SRDP-Compatability
   */
  public toggleSRDPCompatibility(request_id: string, isAlreadySRDP: boolean): Observable<any> {
      const url = `/capability/request/${request_id}/srdp_compatible`;
      let toggleSRDP = isAlreadySRDP ? false : true
      return this.httpClient.post<any>(url, JSON.stringify(toggleSRDP));
  }

  /**
   * Assigns QA Stage 1 and 2 reviewers to a capability request
   * @param requestId  Given request ID, string
   * @param staff  staff member; Staff
   */
  public assignQaStaff(requestId: string, staff: Staff): Observable<Staff> {
    const url = `capability/request/${requestId}/assign/group/${staff.group}/staff/${staff.user_name}`;
    return this.httpClient.post<Staff>(url, { qa_staff: staff });
  }

  /**
   * Cancels a given Capability version and request
   * @param request_id  Given request ID
   * @param version_id  Given version ID
   */
  public cancelRequest(request_id: string, version_id: string): Observable<string> {
    const url = `/capability/request/${request_id}/version/${version_id}/cancel`;
    return this.httpClient.get(url, { responseType: 'text'});
  }

  /**
   * Clears a system message from a request
   * @param requestId Given request id
   * @param msgId Given system message id
   */
  public clearSysMessage(requestId: string, msgId: string): Observable<any> {
    const url = `/capability/request/${requestId}/system_msg/${msgId}/clear`;
    return this.httpClient.post(url, {});
  }
}
